=begin
#Payment Gateway API Specification.

#The documentation here is designed to provide all of the technical guidance required to consume and integrate with our APIs for payment processing. To learn more about our APIs please visit https://docs.firstdata.com/org/gateway.

The version of the OpenAPI document: 21.3.0.20210608.001

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.1.1

=end

require 'date'

module OpenapiClient
  # Merchant supplied tracking numbers.
  class AdditionalDetails
    # Comments for the payment.
    attr_accessor :comments

    # Invoice number.
    attr_accessor :invoice_number

    # Purchase order number.
    attr_accessor :purchase_order_number

    # The operator ID.
    attr_accessor :operator_id

    # The sales system ID.
    attr_accessor :sales_system_id

    # Indicates if the particular transaction is a deferred authorization.
    attr_accessor :ipg_deferred_auth

    # this is highRiskPurchaseIndicator.
    attr_accessor :high_risk_purchase_indicator

    # Provides request information that is necessary to generate receipts.
    attr_accessor :receipts

    # Strong customer authentication exemption type indicator.
    attr_accessor :sca_exemption_type

    # Eight-character Visa merchant identifier (VMID) assigned by Visa, required for trusted merchant and delegated authentication.
    attr_accessor :sca_visa_merchant_id

    # Indicates the indended use of the Account Funding Transaction. For Visa Only.
    attr_accessor :business_application_identifier

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'comments' => :'comments',
        :'invoice_number' => :'invoiceNumber',
        :'purchase_order_number' => :'purchaseOrderNumber',
        :'operator_id' => :'operatorId',
        :'sales_system_id' => :'salesSystemId',
        :'ipg_deferred_auth' => :'ipgDeferredAuth',
        :'high_risk_purchase_indicator' => :'highRiskPurchaseIndicator',
        :'receipts' => :'receipts',
        :'sca_exemption_type' => :'scaExemptionType',
        :'sca_visa_merchant_id' => :'scaVisaMerchantID',
        :'business_application_identifier' => :'businessApplicationIdentifier'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'comments' => :'String',
        :'invoice_number' => :'String',
        :'purchase_order_number' => :'String',
        :'operator_id' => :'String',
        :'sales_system_id' => :'String',
        :'ipg_deferred_auth' => :'Boolean',
        :'high_risk_purchase_indicator' => :'Boolean',
        :'receipts' => :'Array<ReceiptRequestInfo>',
        :'sca_exemption_type' => :'String',
        :'sca_visa_merchant_id' => :'String',
        :'business_application_identifier' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `OpenapiClient::AdditionalDetails` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `OpenapiClient::AdditionalDetails`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'comments')
        self.comments = attributes[:'comments']
      end

      if attributes.key?(:'invoice_number')
        self.invoice_number = attributes[:'invoice_number']
      end

      if attributes.key?(:'purchase_order_number')
        self.purchase_order_number = attributes[:'purchase_order_number']
      end

      if attributes.key?(:'operator_id')
        self.operator_id = attributes[:'operator_id']
      end

      if attributes.key?(:'sales_system_id')
        self.sales_system_id = attributes[:'sales_system_id']
      end

      if attributes.key?(:'ipg_deferred_auth')
        self.ipg_deferred_auth = attributes[:'ipg_deferred_auth']
      end

      if attributes.key?(:'high_risk_purchase_indicator')
        self.high_risk_purchase_indicator = attributes[:'high_risk_purchase_indicator']
      end

      if attributes.key?(:'receipts')
        if (value = attributes[:'receipts']).is_a?(Array)
          self.receipts = value
        end
      end

      if attributes.key?(:'sca_exemption_type')
        self.sca_exemption_type = attributes[:'sca_exemption_type']
      end

      if attributes.key?(:'sca_visa_merchant_id')
        self.sca_visa_merchant_id = attributes[:'sca_visa_merchant_id']
      end

      if attributes.key?(:'business_application_identifier')
        self.business_application_identifier = attributes[:'business_application_identifier']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if !@comments.nil? && @comments.to_s.length > 1024
        invalid_properties.push('invalid value for "comments", the character length must be smaller than or equal to 1024.')
      end

      if !@invoice_number.nil? && @invoice_number.to_s.length > 48
        invalid_properties.push('invalid value for "invoice_number", the character length must be smaller than or equal to 48.')
      end

      if !@purchase_order_number.nil? && @purchase_order_number.to_s.length > 128
        invalid_properties.push('invalid value for "purchase_order_number", the character length must be smaller than or equal to 128.')
      end

      if !@operator_id.nil? && @operator_id.to_s.length > 35
        invalid_properties.push('invalid value for "operator_id", the character length must be smaller than or equal to 35.')
      end

      pattern = Regexp.new(/^\S$|^\S.*\S$/)
      if !@operator_id.nil? && @operator_id !~ pattern
        invalid_properties.push("invalid value for \"operator_id\", must conform to the pattern #{pattern}.")
      end

      if !@sales_system_id.nil? && @sales_system_id.to_s.length > 50
        invalid_properties.push('invalid value for "sales_system_id", the character length must be smaller than or equal to 50.')
      end

      pattern = Regexp.new(/^\S$|^\S.*\S$/)
      if !@sales_system_id.nil? && @sales_system_id !~ pattern
        invalid_properties.push("invalid value for \"sales_system_id\", must conform to the pattern #{pattern}.")
      end

      if !@sca_visa_merchant_id.nil? && @sca_visa_merchant_id.to_s.length > 8
        invalid_properties.push('invalid value for "sca_visa_merchant_id", the character length must be smaller than or equal to 8.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if !@comments.nil? && @comments.to_s.length > 1024
      return false if !@invoice_number.nil? && @invoice_number.to_s.length > 48
      return false if !@purchase_order_number.nil? && @purchase_order_number.to_s.length > 128
      return false if !@operator_id.nil? && @operator_id.to_s.length > 35
      return false if !@operator_id.nil? && @operator_id !~ Regexp.new(/^\S$|^\S.*\S$/)
      return false if !@sales_system_id.nil? && @sales_system_id.to_s.length > 50
      return false if !@sales_system_id.nil? && @sales_system_id !~ Regexp.new(/^\S$|^\S.*\S$/)
      sca_exemption_type_validator = EnumAttributeValidator.new('String', ["Low Value Exemption", "TRA Exemption", "Trusted Merchant Exemption", "SCP Exemption", "Delegated Authentication"])
      return false unless sca_exemption_type_validator.valid?(@sca_exemption_type)
      return false if !@sca_visa_merchant_id.nil? && @sca_visa_merchant_id.to_s.length > 8
      business_application_identifier_validator = EnumAttributeValidator.new('String', ["BANK_INITIATED_TRANSFER", "WALLET_TRANSFER", "ACCOUNT_TO_ACCOUNT", "TOP_OFF", "FUND_TRANSFER", "ZERO_DOLLAR_AUTHORIZATION"])
      return false unless business_application_identifier_validator.valid?(@business_application_identifier)
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] comments Value to be assigned
    def comments=(comments)
      if !comments.nil? && comments.to_s.length > 1024
        fail ArgumentError, 'invalid value for "comments", the character length must be smaller than or equal to 1024.'
      end

      @comments = comments
    end

    # Custom attribute writer method with validation
    # @param [Object] invoice_number Value to be assigned
    def invoice_number=(invoice_number)
      if !invoice_number.nil? && invoice_number.to_s.length > 48
        fail ArgumentError, 'invalid value for "invoice_number", the character length must be smaller than or equal to 48.'
      end

      @invoice_number = invoice_number
    end

    # Custom attribute writer method with validation
    # @param [Object] purchase_order_number Value to be assigned
    def purchase_order_number=(purchase_order_number)
      if !purchase_order_number.nil? && purchase_order_number.to_s.length > 128
        fail ArgumentError, 'invalid value for "purchase_order_number", the character length must be smaller than or equal to 128.'
      end

      @purchase_order_number = purchase_order_number
    end

    # Custom attribute writer method with validation
    # @param [Object] operator_id Value to be assigned
    def operator_id=(operator_id)
      if !operator_id.nil? && operator_id.to_s.length > 35
        fail ArgumentError, 'invalid value for "operator_id", the character length must be smaller than or equal to 35.'
      end

      pattern = Regexp.new(/^\S$|^\S.*\S$/)
      if !operator_id.nil? && operator_id !~ pattern
        fail ArgumentError, "invalid value for \"operator_id\", must conform to the pattern #{pattern}."
      end

      @operator_id = operator_id
    end

    # Custom attribute writer method with validation
    # @param [Object] sales_system_id Value to be assigned
    def sales_system_id=(sales_system_id)
      if !sales_system_id.nil? && sales_system_id.to_s.length > 50
        fail ArgumentError, 'invalid value for "sales_system_id", the character length must be smaller than or equal to 50.'
      end

      pattern = Regexp.new(/^\S$|^\S.*\S$/)
      if !sales_system_id.nil? && sales_system_id !~ pattern
        fail ArgumentError, "invalid value for \"sales_system_id\", must conform to the pattern #{pattern}."
      end

      @sales_system_id = sales_system_id
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] sca_exemption_type Object to be assigned
    def sca_exemption_type=(sca_exemption_type)
      validator = EnumAttributeValidator.new('String', ["Low Value Exemption", "TRA Exemption", "Trusted Merchant Exemption", "SCP Exemption", "Delegated Authentication"])
      unless validator.valid?(sca_exemption_type)
        fail ArgumentError, "invalid value for \"sca_exemption_type\", must be one of #{validator.allowable_values}."
      end
      @sca_exemption_type = sca_exemption_type
    end

    # Custom attribute writer method with validation
    # @param [Object] sca_visa_merchant_id Value to be assigned
    def sca_visa_merchant_id=(sca_visa_merchant_id)
      if !sca_visa_merchant_id.nil? && sca_visa_merchant_id.to_s.length > 8
        fail ArgumentError, 'invalid value for "sca_visa_merchant_id", the character length must be smaller than or equal to 8.'
      end

      @sca_visa_merchant_id = sca_visa_merchant_id
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] business_application_identifier Object to be assigned
    def business_application_identifier=(business_application_identifier)
      validator = EnumAttributeValidator.new('String', ["BANK_INITIATED_TRANSFER", "WALLET_TRANSFER", "ACCOUNT_TO_ACCOUNT", "TOP_OFF", "FUND_TRANSFER", "ZERO_DOLLAR_AUTHORIZATION"])
      unless validator.valid?(business_application_identifier)
        fail ArgumentError, "invalid value for \"business_application_identifier\", must be one of #{validator.allowable_values}."
      end
      @business_application_identifier = business_application_identifier
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          comments == o.comments &&
          invoice_number == o.invoice_number &&
          purchase_order_number == o.purchase_order_number &&
          operator_id == o.operator_id &&
          sales_system_id == o.sales_system_id &&
          ipg_deferred_auth == o.ipg_deferred_auth &&
          high_risk_purchase_indicator == o.high_risk_purchase_indicator &&
          receipts == o.receipts &&
          sca_exemption_type == o.sca_exemption_type &&
          sca_visa_merchant_id == o.sca_visa_merchant_id &&
          business_application_identifier == o.business_application_identifier
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [comments, invoice_number, purchase_order_number, operator_id, sales_system_id, ipg_deferred_auth, high_risk_purchase_indicator, receipts, sca_exemption_type, sca_visa_merchant_id, business_application_identifier].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes, attribute_map = self.class.attribute_map, openapi_types = self.class.openapi_types)
      return nil unless attributes.is_a?(Hash)
      openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        klass = OpenapiClient.const_get(type)
        if klass.respond_to?(:openapi_discriminator_name)
          klass = OpenapiClient.const_get(value[klass.attribute_map[klass.openapi_discriminator_name]])
        end
 
        klass.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash(attribute_map = self.class.attribute_map, openapi_nullable = Set.new([]))
      hash = {}
      attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end
        
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
