=begin
#Payment Gateway API Specification.

#The documentation here is designed to provide all of the technical guidance required to consume and integrate with our APIs for payment processing. To learn more about our APIs please visit https://docs.firstdata.com/org/gateway.

The version of the OpenAPI document: 21.3.0.20210608.001

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.1.1

=end

require 'date'

module OpenapiClient
  # Additional version-dependent information used to decrypt and verify the payment.
  class EncryptedApplePayHeader
    # Merchant supplied information about the payment request. Contains Base64-encoded SHA256 hash of the applicationData property of the original PKPaymentRequest. Note - applicationData from PaymentData of PKPaymentToken Refer to Apple Pay documentation.
    attr_accessor :application_data_hash

    # Temporary key for generating shared secret from a device.
    attr_accessor :ephemeral_public_key

    # Hash of the X.509 encoded public key bytes of the merchant’s certificate.
    attr_accessor :public_key_hash

    # Transaction identifier, generated on the device.
    attr_accessor :transaction_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'application_data_hash' => :'applicationDataHash',
        :'ephemeral_public_key' => :'ephemeralPublicKey',
        :'public_key_hash' => :'publicKeyHash',
        :'transaction_id' => :'transactionId'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'application_data_hash' => :'String',
        :'ephemeral_public_key' => :'String',
        :'public_key_hash' => :'String',
        :'transaction_id' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `OpenapiClient::EncryptedApplePayHeader` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `OpenapiClient::EncryptedApplePayHeader`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'application_data_hash')
        self.application_data_hash = attributes[:'application_data_hash']
      end

      if attributes.key?(:'ephemeral_public_key')
        self.ephemeral_public_key = attributes[:'ephemeral_public_key']
      end

      if attributes.key?(:'public_key_hash')
        self.public_key_hash = attributes[:'public_key_hash']
      end

      if attributes.key?(:'transaction_id')
        self.transaction_id = attributes[:'transaction_id']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @ephemeral_public_key.nil?
        invalid_properties.push('invalid value for "ephemeral_public_key", ephemeral_public_key cannot be nil.')
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if @ephemeral_public_key !~ pattern
        invalid_properties.push("invalid value for \"ephemeral_public_key\", must conform to the pattern #{pattern}.")
      end

      if @public_key_hash.nil?
        invalid_properties.push('invalid value for "public_key_hash", public_key_hash cannot be nil.')
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if @public_key_hash !~ pattern
        invalid_properties.push("invalid value for \"public_key_hash\", must conform to the pattern #{pattern}.")
      end

      if @transaction_id.nil?
        invalid_properties.push('invalid value for "transaction_id", transaction_id cannot be nil.')
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if @transaction_id !~ pattern
        invalid_properties.push("invalid value for \"transaction_id\", must conform to the pattern #{pattern}.")
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @ephemeral_public_key.nil?
      return false if @ephemeral_public_key !~ Regexp.new(/^(?!\s*$).+/)
      return false if @public_key_hash.nil?
      return false if @public_key_hash !~ Regexp.new(/^(?!\s*$).+/)
      return false if @transaction_id.nil?
      return false if @transaction_id !~ Regexp.new(/^(?!\s*$).+/)
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] ephemeral_public_key Value to be assigned
    def ephemeral_public_key=(ephemeral_public_key)
      if ephemeral_public_key.nil?
        fail ArgumentError, 'ephemeral_public_key cannot be nil'
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if ephemeral_public_key !~ pattern
        fail ArgumentError, "invalid value for \"ephemeral_public_key\", must conform to the pattern #{pattern}."
      end

      @ephemeral_public_key = ephemeral_public_key
    end

    # Custom attribute writer method with validation
    # @param [Object] public_key_hash Value to be assigned
    def public_key_hash=(public_key_hash)
      if public_key_hash.nil?
        fail ArgumentError, 'public_key_hash cannot be nil'
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if public_key_hash !~ pattern
        fail ArgumentError, "invalid value for \"public_key_hash\", must conform to the pattern #{pattern}."
      end

      @public_key_hash = public_key_hash
    end

    # Custom attribute writer method with validation
    # @param [Object] transaction_id Value to be assigned
    def transaction_id=(transaction_id)
      if transaction_id.nil?
        fail ArgumentError, 'transaction_id cannot be nil'
      end

      pattern = Regexp.new(/^(?!\s*$).+/)
      if transaction_id !~ pattern
        fail ArgumentError, "invalid value for \"transaction_id\", must conform to the pattern #{pattern}."
      end

      @transaction_id = transaction_id
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          application_data_hash == o.application_data_hash &&
          ephemeral_public_key == o.ephemeral_public_key &&
          public_key_hash == o.public_key_hash &&
          transaction_id == o.transaction_id
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [application_data_hash, ephemeral_public_key, public_key_hash, transaction_id].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes, attribute_map = self.class.attribute_map, openapi_types = self.class.openapi_types)
      return nil unless attributes.is_a?(Hash)
      openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        klass = OpenapiClient.const_get(type)
        if klass.respond_to?(:openapi_discriminator_name)
          klass = OpenapiClient.const_get(value[klass.attribute_map[klass.openapi_discriminator_name]])
        end
 
        klass.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash(attribute_map = self.class.attribute_map, openapi_nullable = Set.new([]))
      hash = {}
      attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end
        
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
